<?php
/*
Plugin Name: Ultimate Member - MailChimp By ScriptYab.com
Plugin URI: http://ultimatemember.com/
Description: Integrates Ultimate Member with MailChimp newsletters.
Version: 2.1.1
Author: Ultimate Member
Author URI: http://ultimatemember.com/
*/

require_once( ABSPATH.'wp-admin/includes/plugin.php' );

$plugin_data = get_plugin_data( __FILE__ );

define( 'um_mailchimp_url',plugin_dir_url(__FILE__ ));
define( 'um_mailchimp_path',plugin_dir_path(__FILE__ ));
define( 'um_mailchimp_plugin', plugin_basename( __FILE__ ) );
define( 'um_mailchimp_extension', $plugin_data['Name'] );
define( 'um_mailchimp_version', $plugin_data['Version'] );
define( 'um_mailchimp_textdomain', 'um-mailchimp' );

define( 'um_mailchimp_requires', '2.0.1' );

function um_mailchimp_plugins_loaded() {
	$locale = ( get_locale() != '' ) ? get_locale() : 'en_US';
	load_textdomain( um_mailchimp_textdomain, WP_LANG_DIR . '/plugins/' . um_mailchimp_textdomain . '-' . $locale . '.mo' );
    load_plugin_textdomain( um_mailchimp_textdomain, false, dirname( plugin_basename( __FILE__ ) ) . '/languages/' );
}
add_action( 'plugins_loaded', 'um_mailchimp_plugins_loaded', 0 );

add_action( 'plugins_loaded', 'um_mailchimp_check_dependencies', -20 );

if ( ! function_exists( 'um_mailchimp_check_dependencies' ) ) {
    function um_mailchimp_check_dependencies() {
        if ( ! defined( 'um_path' ) || ! file_exists( um_path  . 'includes/class-dependencies.php' ) ) {
            //UM is not installed
            function um_mailchimp_dependencies() {
                echo '<div class="error"><p>' . sprintf( __( 'The <strong>%s</strong> extension requires the Ultimate Member plugin to be activated to work properly. You can download it <a href="https://wordpress.org/plugins/ultimate-member">here</a>', 'um-mailchimp' ), um_mailchimp_extension ) . '</p></div>';
            }

            add_action( 'admin_notices', 'um_mailchimp_dependencies' );
        } else {

            if ( ! function_exists( 'UM' ) ) {
                require_once um_path . 'includes/class-dependencies.php';
                $is_um_active = um\is_um_active();
            } else {
                $is_um_active = UM()->dependencies()->ultimatemember_active_check();
            }

            if ( ! $is_um_active ) {
                //UM is not active
                function um_mailchimp_dependencies() {
                    echo '<div class="error"><p>' . sprintf( __( 'The <strong>%s</strong> extension requires the Ultimate Member plugin to be activated to work properly. You can download it <a href="https://wordpress.org/plugins/ultimate-member">here</a> or <a href="http://tiny.cc/ultimate-member">here</a>', 'um-mailchimp' ), um_mailchimp_extension ) . '</p></div>';
                }

                add_action( 'admin_notices', 'um_mailchimp_dependencies' );

            } elseif ( true !== UM()->dependencies()->compare_versions( um_mailchimp_requires, um_mailchimp_version, 'mailchimp', um_mailchimp_extension ) ) {
                //UM old version is active
                function um_mailchimp_dependencies() {
                    echo '<div class="error"><p>' . UM()->dependencies()->compare_versions( um_mailchimp_requires, um_mailchimp_version, 'mailchimp', um_mailchimp_extension ) . '</p></div>';
                }

                add_action( 'admin_notices', 'um_mailchimp_dependencies' );

            } else {
                require_once um_mailchimp_path . 'includes/core/um-mailchimp-init.php';
            }
        }
    }
}


register_activation_hook( um_mailchimp_plugin, 'um_mailchimp_activation_hook' );
function um_mailchimp_activation_hook() {
    //first install
    $version = get_option( 'um_mailchimp_version' );
    if ( ! $version )
        update_option( 'um_mailchimp_last_version_upgrade', um_mailchimp_version );

    if ( $version != um_mailchimp_version )
        update_option( 'um_mailchimp_version', um_mailchimp_version );


    //run setup
    if ( ! class_exists( 'um_ext\um_mailchimp\core\Mailchimp_Setup' ) )
        require_once um_mailchimp_path . 'includes/core/class-mailchimp-setup.php';

    $mailchimp_setup = new um_ext\um_mailchimp\core\Mailchimp_Setup();
    $mailchimp_setup->run_setup();
}